// version 2.6 //

var drmType         = "PROIDIOM";
var sessionId       = "4C47453436333133303132383039313231303138";

const udpPort       = 6868              // Port to open for UDP commands

var maxRuntime      = 26;               // Number of continous hours before a forced reboot

var failsafe;                           // Timer to initiate a failsafe reboot after the max runtime
var portalTimer;                        // Timer for portal time out
var powerMode;                          // Place holder for the power mode warm or always on

var tv              = {};               // Empty object to hold the current settings from the TV

var tuneSuccessful  = 0;                // Flag to know our tune was successful

var debugging       = "false";          // Enable printing to the screen


// Get functions //

function GetProperty(key) {
    hcap.property.getProperty(
        {
            "key" : key,
            "onSuccess" : function(s) {
                tv[key] = s.value;
                print( { "name" : "GetProperty()", "message" : key +" - " + tv[key] } );
            },
            "onFailure" : function(f) {
                print( { "name" : "GetProperty()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetIPAddress() {
    hcap.network.getNetworkInformation(
        {
            "onSuccess" : function(s) {
                tv.ip_address = s.ip_address;
                print( { "name" : "GetIPAddress()", "message" : tv.ip_address } );
            },
            "onFailure" : function(f) {
                print( { "name" : "GetIPAddress()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetProCentricServer() {
    hcap.system.getProcentricServer(
        {
            "onSuccess" : function(p) {
                var proCentricServer = {"mode" : p.mode,"media" : p.media};
        
                if (p.media === "rf") {
                    proCentricServer.rfDataChannel              = p.rfServer.dataChannelNumber;
                }
                else if (p.media === "ip") {
                    if (p.ipServer.type === "ip") {
                        proCentricServer.ipServerAddress        = p.ipServer.ip;
                        proCentricServer.ipServerPort           = p.ipServer.port;
                    }
                    else if (p.ipServer.type === "domain_name") {
                        proCentricServer.ipServerDomainName     = p.ipServer.domainName;
                        proCentricServer.ipServerPort           = p.ipServer.port;
                    }
                    tv.proCentricServer     = proCentricServer;

                    if (tv.proCentricServer.media === "rf") {
                        print( { "name" : "GetProcentricServer()", "message" : tv.proCentricServer.rfDataChannel } );
                    }
                    if (tv.proCentricServer.media === "ip") {
                        print( { "name" : "GetProcentricServer()", "message" : tv.proCentricServer.ipServerAddress + ":" + tv.proCentricServer.ipServerPort } );
                    }
                }

                setDefaults();
            },
            "onFailure" : function(f) {
                print( { "name" : "GetProcentricServer()", "error" : f.errorMessage } );
                setDefaults();
            }
        }
    );
}    
function GetVideoInfo() {
    hcap.video.getVideoSize(
        {  
            "onSuccess" : function(s) {
                tv.video            = {};
                tv.video.posX          = s.x;
                tv.video.posY          = s.y;
                tv.video.width         = s.width;
                tv.video.height        = s.height;
                print( { "name" : "GetVideoInfo()", "message" : tv.video.width + "x" + tv.video.height + " | " + tv.video.posX + "," + tv.video.posY } );
            },
            "onFailure" : function(f) {
                print( { "name" : "GetVideoInfo()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetCurrentChannel() {
    hcap.channel.getCurrentChannel(
        {
            "onSuccess" : function(s) {
                tv.currentChannel           = {};
                
                if (s.majorNumber !== undefined) {
                    tv.currentChannel.major     = s.majorNumber;
                    tv.currentChannel.minor     = s.minorNumber;
                
                    print( { "name" : "GetCurrentChannel()", "message" : tv.currentChannel.major + "." + tv.currentChannel.minor } );
                }
                if (s.ip !== undefined) {
                    tv.currentChannel.ip        = s.ip;
                    tv.currentChannel.port      = s.port;
                
                    print( { "name" : "GetCurrentChannel()", "message" : tv.currentChannel.ip + ":" + tv.currentChannel.port } );
                }
            },
            "onFailure" : function(f) {
                print( { "name" : "GetCurrentChannel()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetPowerMode() {
    hcap.power.getPowerMode(
        {
            "onSuccess" : function(s) {
                tv.power_mode =  s.mode
                print( { "name" : "GetPowerMode()", "message" : tv.power_mode } );
            },
            "onFailure" : function(f) {
                print( { "name" : "GetPowerMode()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetLocalTime() {
    hcap.time.getLocalTime(
        {
            "onSuccess" : function(s) {
                var timestamp;

                tv.dateTime = {};

                tv.dateTime.year        = s.year;
                tv.dateTime.month       = s.month;
                tv.dateTime.day         = s.day;
                tv.dateTime.hour        = s.hour;
                tv.dateTime.minute      = s.minute;
                tv.dateTime.second      = s.second;
                tv.dateTime.offset      = s.gmtOffsetInMinute;
                tv.dateTime.dst         = s.isDaylightSaving;
                
                timestamp               = "<br> TIME / DATE INFO <br> "
                                        + "Current Date &emsp;  " + tv.dateTime.month + "/" + tv.dateTime.day + "/" + tv.dateTime.year  + "<br>"
                                        + "Current Time &emsp;  " + ZeroPad(tv.dateTime.hour, 2) + ":" + ZeroPad(tv.dateTime.minute, 2) + ":" + ZeroPad(tv.dateTime.second, 2) + "<br>"
                                        + "GMT Offset &emsp;    " + tv.dateTime.offset/60 + "<br>"
                                        + "DST &emsp;           " + tv.dateTime.dst;

                tv.timestamp            = timestamp;

                print( { "name" : "", "message" : tv.timestamp } );
            },
            "onFailure" : function(f) {
                print( { "name" : "GetLocalTime()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetCpuTime() {
    hcap.time.getCpuTime(
        {
            "onSuccess" : function(param) {
                var time, seconds, days, hours, minutes, uptime;

                time                = param.cpuTime;
                
                seconds             = Math.floor(time/1000);
                days                = Math.floor(seconds/3600/24);
                seconds            -= days*3600*24;
                hours               = Math.floor(seconds/3600);
                seconds            -= hours*3600;
                minutes             = Math.floor(seconds/60);
                seconds            -= minutes*60;

                uptime              = "<br> UPTIME " + days + "d " + ZeroPad(hours, 2) + "h " + ZeroPad(minutes, 2) + "m " + ZeroPad(seconds, 2) + "s <br>";

                tv.uptime           = uptime;

                print( { "name" : "", "message" : tv.uptime } );
            },
            "onFailure" : function(f) {
                print( { "name" : "GetCpuTime()", "error" : f.errorMessage } );
            }
        }
    );
}
function GetTVname() {
    var name = localStorage.getItem("name");

    if (name != null) {
        tv.name = name;
        document.getElementById("tvname").innerHTML = tv.name;
    }
}


// Set functions //

function SetProperty(key, value) {
    hcap.property.setProperty(
        {
            "key" : key,
            "value" : value,
            "onSuccess" : function() {
                print( { "name" : "SetProperty()", "message" : key + " set to " + value } );
            },
            "onFailure" : function(f) {
                print( { "name" : "SetProperty()", "error" : f.errorMessage } );
            }
        }
    );
}
function SetPowerMode() {
    var powerMode = localStorage.getItem("LocalPowerMode");         // Get the saved settings if we have it
        
    if (powerMode == null) {                                        // No previous power mode setting so we need to set it to a default
        powerMode = 2;                                              // Set to warm mode
        localStorage.setItem("LocalPowerMode", powerMode);          // Write to localstorage
    }
    
    hcap.property.setProperty(
        {
            "key" : "instant_power",
            "value" : powerMode,
            "onSuccess" : function() {
                print( { "name" : "SetPowerMode()", "message" : "Power Mode set to " + powerMode } );

            },
            "onFailure" : function(f) {
                print( { "name" : "SetPowerMode()", "error" : f.errorMessage } );
            }
        }
    );

    if (powerMode == "2") {
        document.getElementById('AC_On=2').checked = true;
    }
    else {
        document.getElementById('AC_On=0').checked = true;
    }
}
function setBrowserDebug(value) {
    hcap.system.setBrowserDebugMode(
        {
            "debugMode" : value,
            "onSuccess" : function() {
                print( { "name" : "setBrowserDebug()", "message" : "SUCCESS" } );
                poll();
            },
            "onFailure" : function(f) {
                print( { "name" : "setBrowserDebug()", "error" : f.errorMessage } );
                poll();
            }
        }
    );
}
function setDebugMode() {
    if (!localStorage.debugMode) {
        localStorage.debugMode = debugging;
    }

    switch (localStorage.getItem("debugMode")) {
        case "true":
            debugging = "true";

            Hide( [ "settings_window", "tvname_window"  ] );
            Show( ["logo_image", "header", "main"       ] );
    
            document.body.style.backgroundImage = "";
            document.body.style.backgroundColor = "#fff";

            $("#debug-button").removeAttr("style");
            $("#debug-button").text("Debugging Enabled");
            
            break;
        
        case "false":
            debugging = "false";

            Hide( [ "logo_image", "header", "main", "settings_window", "tvname_window" ] );
            
            document.body.style.backgroundImage = "url('tv:')";
            document.body.style.backgroundColor = "transparent";

            $("#debug-button").attr("style", "background-color: #ff0000");
            $("#debug-button").text("Debugging Disabled");

            break;
    }
}
function SetAspectRatio(value) {
    hcap.property.setPictureProperty(
        {
            "key"       : hcap.property.PicturePropertyKey.ASPECT_RATIO,
            "value"     : value,
            "onSuccess" : function(s) {
                print( { "name" : "SetAspectRatio()", "message" : value + " " + s.result } );
            }, 
            "onFailure" : function(f) {
                print( { "name" : "SetAspectRatio()", "error" : f.errorMessage } );
            }
        }
    );
}
function SetHCAPmode(value) {
   switch (value) {
        case "0":
            hcap.mode.setHcapMode(
                {
                    "mode" : hcap.mode.HCAP_MODE_0,
                    "onSuccess" : function() {
                        print( { "name" : "SetHCAPmode()", "message" : "Mode " + value } )
                    }, 
                    "onFailure" : function(f) {
                        print( { "name" : "SetHCAPmode()", "error" : f.errorMessage } );
                    }
                }
            );
            
            break;

        case "1":
            hcap.mode.setHcapMode(
                {
                    "mode" : hcap.mode.HCAP_MODE_1,
                    "onSuccess" : function() {
                        print( { "name" : "SetHCAPmode()", "message" : "Mode " + value  } )
                    }, 
                    "onFailure" : function(f) {
                        print( { "name" : "SetHCAPmode()", "error" : f.errorMessage } );
                    }
                }
            );    
    }
}
function SetTVname(tvName) {
    print({"name" : "SetTVname()", "message" : "TV name change requested...."})

    var tvId = tvName;      // TV:DSCHD
    var i;
    
    for (i = 0; i < tvId.length; ++i) {
        if (tvId.substring(i, i+1) == ':') {
            tvId = tvId.substring(0, i);
        }
    }

    tv.name = tvId;

    localStorage.setItem("name", tv.name);

    document.getElementById("tvname").innerHTML = tv.name;

    print({"name" : "SetTVname()", "message" : "New TV name = " + tv.name})
}
function SetVolume(value) {
    hcap.volume.setVolumeLevel(
        {
            "level" : value, 
            "onSuccess" : function() {
                print( { "name" : "Volume", "message" : " set to " + value } )
            }, 
            "onFailure" : function(f) {
                print( { "name" : "SetVolume()", "error" : f.errorMessage } );
            }
        }
    );
}
function SetFailsafe() {
    switch (localStorage.getItem("failsafe")) {
        case "1" :
            $("#failsafe-button").removeAttr("style");
            $("#failsafe-button").text("Failsafe Enabled");

            failsafe = CheckRuntime();

            break;

        case "0" :
            $("#failsafe-button").attr("style", "background-color: #ff0000");
            $("#failsafe-button").text("Failsafe Disabled");

            clearTimeout(failsafe);
            print( { "name" : "Failsafe Reboot", "warning": "Disabled.  TV will NOT reboot." } );

            break;
    } 
}


// Tune Functions //

function TuneRF(major, minor) {
    tuneSuccessful = 0;

    if (drmType) {
        hcap.channel.requestChangeCurrentChannel(
            {
                "channelType"       : hcap.channel.ChannelType.RF,
                "majorNumber"       : major,
                "minorNumber"       : minor,
                "rfBroadcastType"   : hcap.channel.RfBroadcastType.CABLE,
                "drmType"           : drmType,
                "sessionId"         : sessionId,
                "onSuccess":function() {
                    print( { "name" : "", "message" : "Encrypted RF Channel Change Request Successful" } );
                },
                "onFailure" : function(f) {
                    print( { "name" : "TuneRF()", "error" : f.errorMessage } );
                }
            }
        );
    }
    else {
        hcap.channel.requestChangeCurrentChannel(
            {
                "channelType"       :hcap.channel.ChannelType.RF,
                "majorNumber"       :major,
                "minorNumber"       :minor,
                "rfBroadcastType"   :hcap.channel.RfBroadcastType.CABLE,
                "onSuccess":function() {
                    print( { "name" : "", "message" : "RF Channel Change Request Successful" } );
                },
                "onFailure" : function(f) {
                    print( { "name" : "TuneRF()", "error" : f.errorMessage } );
                }
            }
        );
    }
}
function TuneIP(ip, port) {
    tuneSuccessful = 0;

    if (drmType) {
        hcap.channel.requestChangeCurrentChannel(
            {
                "channelType"           : hcap.channel.ChannelType.IP,
                "ip"                    : ip,
                "port"                  : port,
                "ipBroadcastType"       : hcap.channel.IpBroadcastType.UDP,
                //"audioPid"              : hcap.channel.NO_STREAM_PID,
                //"videoPid"              : 224,
                //"audioPid"              : 189,
                "drmType"               : drmType,
                "sessionId"             : sessionId,
                "onSuccess":function() {
                    print( { "name" : "", "message" : "Encrypted IP Channel Change Request Successful" } );
                },
                "onFailure" : function(f) {
                    print( { "name" : "TuneIP()", "error" : f.errorMessage } );
                }
            }
        );
    }
    else {
        hcap.channel.requestChangeCurrentChannel(
            {
                "channelType"           : hcap.channel.ChannelType.IP,
                "ip"                    : ip,
                "port"                  : port,
                "ipBroadcastType"       : hcap.channel.IpBroadcastType.UDP,
                //"audioPid"              : hcap.channel.NO_STREAM_PID,
                //"videoPid"              : 224,
                //"audioPid"              : 189,
                "onSuccess":function() {
                    print( { "name" : "", "message" : "IP Channel Change Request Successful" } );
                },
                "onFailure" : function(f) {
                    print( { "name" : "TuneIP()", "error" : f.errorMessage } );
                }
            }
        );
    } 
}
function Retune() {
    switch ( localStorage.getItem("StorType") ) {                           // retune the last channel to allow decryption    
        case "RF":
            var major           = Number(localStorage.getItem("StorMajor"));
            var minor           = Number(localStorage.getItem("StorMinor"));
            
            TuneRF(major, minor);
            
            break;
        
        case "IP":
            var ip              = localStorage.getItem("StorIp");
            var port            = Number(localStorage.getItem("StorPort"));
            
            TuneIP(ip, port);

            break;
    
        default:
            var ip              = "226.1.1.1";
            var port            = 5000;

            localStorage.setItem("StorType", "IP");                         // Store the type of last channel
            localStorage.setItem("StorIp", ip);                             // save the IP to restore on reboot
            localStorage.setItem("StorPort", port);                         // save the port to restore on reboot

            TuneIP(ip, port);

            print( { "name" : "", "message" : "Initializing to default IP channel - " + ip + ":" + String(port) } );
    }
}


// Utility Functions //

function print(item) {
    var message     = item.message;
    var error       = item.error;
    var warning     = item.warning

    if (message != undefined) {
        console.info(item.name + " : " + item.message);
        
        if (localStorage.debugMode === "true") {
            $("#debug").append(item.name + " ");
            $("#debug").append(item.message);
            $("#debug").append("<br>");
            $("#debug_container").scrollTop($("#debug").height());
        }
    }
    if (warning != undefined) {
        console.warn(item.name + " :: WARNING : " + item.warning);
        
        if (localStorage.debugMode === "true") {
            $("#debug").append( item.name + " :: WARNING : " + item.warning + "<br>" );
            $("#debug_container").scrollTop($("#debug").height());
        }
    }
    if (error != undefined) {
        console.error(item.name + " :: ERROR : " + item.error);
        
        if (localStorage.debugMode === "true") {
            $("#debug").append( item.name + " :: ERROR : " + item.error + "<br>" );
            $("#debug_container").scrollTop($("#debug").height());
        }
    }
}
function LocalStorageTest() {
    localStorage.setItem("test", "Local Storage Ready... ");
    
    var test = localStorage.getItem("test");
    
    if (test){
        $("#debug").append(test);
        console.info(test);

        return true;
    }
    else {
        $("#debug").append("Local Storage Not Ready <br>");
        console.warn("Local Storage Not Ready");

        return false;
    }
}
function ZeroPad(num, places) {
    var zero = places - num.toString().length + 1;
    
    return Array(+(zero > 0 && zero)).join("0") + num;
}
function OpenUDP(value) {
    hcap.socket.openUdpDaemon(
        {
            "port"      : value,
            "onSuccess" : function() {
                print( { "name" : "OpenUDP()", "message" : "UDP port open on " + value } );
                restore();
            },
            "onFailure":function(f) {
                print( { "name" : "OpenUDP()", "error" : f.errorMessage } );
                restore();
            }
        }
    );
}
function CloseUDP(value) {
   hcap.socket.closeUdpDaemon(
        {
            "port"  : value,
            "onSuccess" : function() {
                print( { "name" : "CloseUDP()", "warning" : "UDP Socket Closed on port " + value } );
            },
            "onFailure":function(f) {
                print( { "name" : "CloseUDP()", "error" : f.errorMessage } );
            }
        }
    );
}
function RegisterTV() {
    var url;

    var ip      = localStorage.getItem("StorIp");
    var port    = localStorage.getItem("StorPort");

    if ( tv.proCentricServer.ipServerAddress ) {
        url   = "http://"           + tv.proCentricServer.ipServerAddress   +
                "/cgi-bin/webcmd?"  +
                "screen=lgtvInit"   +
                "&tvAddress="       + tv.ip_address                         +
                "&RoomNum="         + tv.room_number                        +
                "&tuneIp="          + ip                                    +
                "&tunePort="        + port                                  ;
    }
    else {
        return;
    }

    $.get(url)
        .done(
            function() {
                print( { "name"  : "RegisterTV()", "message" : "TV registration successful"  } );
            }
        )
        .fail(
            function() {
                print( { "name"  : "RegisterTV()", "error" : "Unable to register TV"        } );
            }
        );
}
function CheckRuntime() {
    print( { "name" : "Failsafe Reboot", "message": "timer started.  TV will reboot in " + maxRuntime + " hours." } );

    const timer = setTimeout (
        function() {
            Hide( [ "main", "header", "logo" ] );                                             // hide the main window
            Show( [ "reboot_message"         ] );                                             // show the reboot warning

            document.body.style.backgroundColor = "transparent";
            document.body.style.backgroundImage = "url('tv:')";
            
            print ({ "name" : "Failsafe Reboot", "warning" : "timer expired.  TV will reboot in 3...2...1..." } )
            
            setTimeout(function() { reboot(); }, 3000);                                     // wait 3 sceconds before rebooting
        },
        maxRuntime * 3600000                                                                // maxRuntime in hours converted to milliseconds (Y x 3600000)
    );

    return timer;
}
function TimerPortal(seconds) {
    if(portalTimer != null)
        clearTimeout(portalTimer);
    
        portalTimer = setTimeout (function() {
            portalTimer = null;
            HidePortalWindow();
        }, seconds*1000);
}
function reboot() {
      print( { "name" : " ", "message" : "Rebooting..." } );
    
    hcap.power.reboot(
        {
            "onSuccess":function() {
                console.info("Reboot Initiated");
                CloseUDP(udpPort);
            },
            "onFailure":function(f) {
                print( { "name" : "Reboot", "error": f.errorMessage })
            }
        }
    );
}
function SetPower() {
    var checked     = document.querySelector('[name="ACsetting"]:checked');     // Get the value selected
    
    powerMode       = checked.value;
    
    localStorage.setItem("LocalPowerMode", powerMode)                          // Store it as the new value

    SetPowerMode();
}
function ToggleDebug() {
    switch (localStorage.getItem("debugMode")) {
        case "true":
            localStorage.setItem("debugMode", "false");

            $("#debug-button").attr("style", "background-color: #ff0000");
            $("#debug-button").text("Debugging Disabled");
            
            $("#debug").append("Debugging off <br>");
            $("#debug_container").scrollTop($("#debug").height());

            break;
       
        case "false":
            localStorage.setItem("debugMode", "true");


            $("#debug-button").removeAttr("style");
            $("#debug-button").text("Debugging Enabled");

            $("#debug").append("Debugging on <br>");
            $("#debug_container").scrollTop($("#debug").height());

            break;
        
        default:
            localStorage.setItem("debugMode", "false");

            $("#debug-button").attr("style", "background-color: #ff0000");
            $("#debug-button").text("Debugging Disabled");

            $("#debug").append("Set debugging to default off <br>");
            $("#debug_container").scrollTop($("#debug").height());
            
            break;
    }
}
function ToggleFailsafe() {
   switch (localStorage.getItem("failsafe")) {
        case "0" :
            localStorage.setItem("failsafe", "1");

            $("#failsafe-button").removeAttr("style");
            $("#failsafe-button").text("Failsafe Enabled");

            SetFailsafe();

            break;

        case "1" :
            localStorage.setItem("failsafe", "0");

            $("#failsafe-button").attr("style", "background-color: #ff0000");
            $("#failsafe-button").text("Failsafe Disabled");

            SetFailsafe();

            break;
    }
}


// Page View Functions //

function Hide(ids) {
    for (var i = 0; i < ids.length; i++) {
        document.getElementById(ids[i]).style.display = "none";
    }
}
function Show(ids) {
    for (var i = 0; i < ids.length; i++) {
        document.getElementById(ids[i]).style.display = "block";
    }
}


function HidePortalWindow() {
    localStorage.portalUp = 0;

    if(localStorage.getItem("showName") != '1' || tv.name == null) {
        hcap.mode.setHcapMode({"mode" : hcap.mode.HCAP_MODE_0});
        return;
    }

    Hide(   [ "logo_image",   "header",   "main", "settings_window" ] );
    Show(   [ "tvname_window"                                       ] );

    document.body.style.backgroundColor = "transparent";
    document.body.style.backgroundImage = "url('tv:')";

    hcap.mode.setHcapMode({"mode" : hcap.mode.HCAP_MODE_1});
}
function ShowPortalWindow() {
    localStorage.portalUp   = 1;
    idIndex                 = 0;
    
    Hide( [ "settings_window", "tvname_window"                              ] );
    Show( [ "logo_image", "header", "main", "debug_container"               ] );

    document.body.style.backgroundImage = "";
    document.body.style.backgroundColor = "#fff";

    $("#reboot-button").attr    ( "class", "btn"                );
    $("#register-button").attr  ( "class", "btn"                );
    $("#settings-button").attr  ( "class", "btn"                );
    $("#reload-button").attr    ( "class", "btn btn-success"    );
    $("#reload-button").focus();
    
    hcap.video.setOsdTransparencyLevel({ "level" : 100 });
    
    hcap.mode.setHcapMode({"mode" : hcap.mode.HCAP_MODE_1});
    
    // Clear screen
    $("#debug").replaceWith('<p id="debug"></p>');

    // get the current time
    try         { GetLocalTime();                                               }
    catch(err)  { print( { "name"  : "GetLocalTime()", "error" : err } )        }

    // get the current uptime
    try         { GetCpuTime();                                                 }
    catch(err)  { print( { "name"  : "GetCpuTime()", "error" : err } )          }
    
    // timeout the portal
    try         { TimerPortal(90);                                              }
    catch(err)  { print( { "name"  : "TimerPortal()", "error" : err } )         }

    $("#debug").append("<b>    XAIT Version - " + tv.xait_version                                                              + "</b> <br> <br>"    ); 
    $("#debug").append("            TV Name - " + tv.name                                                                      + "<br>          "    );
    $("#debug").append("      TV IP Address - " + tv.ip_address                                                                + "<br>          "    );
    $("#debug").append("     TV Room Number - " + tv.room_number                                                               + "<br>          "    );
    $("#debug").append(" Pro:Centric Server - " + tv.proCentricServer.ipServerAddress + ":" + tv.proCentricServer.ipServerPort + "<br>          "    );
    $("#debug").append(" Display Resolution - " + tv.display_resolution                                                        + "<br>          "    );
    $("#debug").append("         Video Size - " + tv.video.width + "x" + tv.video.height                                       + "<br> <br>     "    );
    $("#debug").append("    Current Channel - " + tv.currentChannel.ip + ":" + tv.currentChannel.port                          + "<br>          "    );

    if ( localStorage.getItem("debugMode") === "false" ) {
        setTimeout(
            function() {
                $("#debug").append(               tv.timestamp                                                                 + "<br> <br>     "    );
                $("#debug").append(               tv.uptime                                                                    + "<br> <br>     "    );
            }, 500
        );
    }
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      This runs on page load.  It sets debugging functions, polls the TV for its current data,                                            //
//      sets defaults, and then performs a reune.  It is chained in the callback functions of the hcap calls,                               //
//      so it is importatnt that the program flow is not changed.                                                                           //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function initialize() {     // STEP 1
    var ready;

    do { ready = LocalStorageTest(); } while (ready != true)
    
    print( { "name" : " ", "message" : "Initializing..." } );

    // set the debug mode
    try         { setDebugMode();                                                           }
    catch(err)  { print( { "name"  : "setDebugMode()", "error" : err } )                    }

    // Disable the UI overlays
    try         { SetProperty("tv_channel_attribute_floating_ui", "0");                     }
    catch(err)  { print( { "name"  : "GetPowerMode()", "error" : err } )                    }

    try         { SetProperty("boot_sequence_option", "1");                                 }
    catch(err)  { print( { "name"  : "SetProperty(boot_sequence_option)", "error" : err } ) }


    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //      Turns on browser debug.  This must be the last function in the routine, as it calls poll(),                      //
    //      which is the next step in the initialization routine.                                                           //
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    try         { setBrowserDebug(true);                                                    }
    catch(err)  { print( { "name"  : "setBrowserDebug()", "error" : err } )                 }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      This is called in the onSuccess callback of setBrowserDebug().  It gathers that current data from the TV.                           //
//      It is chained in the callback functions of the hcap calls.                                                                          //
//      It is importatnt that the program flow is not changed.                                                                              //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function poll() {           // STEP 2
    // get the current power mode
    try         { GetPowerMode();                                                           }
    catch(err)  { print( { "name"  : "GetPowerMode()", "error" : err } )                    }

    // get the ip address of the tv
    try         { GetIPAddress();                                                           }
    catch(err)  { print( { "name"  : "GetIPAddress()", "error" : err } )                    }

    // get room name
    try         { GetProperty("room_number");                                               }
    catch(err)  { print( { "name"  : "GetProperty(room_number)", "error" : err } )          }

    // get the xait version number
    try         { GetProperty("xait_version");                                              }
    catch(err)  { print( { "name"  : "GetProperty(xait_version)", "error" : err } )         }

    // get the display resolution
    try         { GetProperty("display_resolution");                                        }
    catch(err)  { print( { "name"  : "GetProperty(display_resolution)", "error" : err } )   }

    // get the current video size and position
    try         { GetVideoInfo();                                                           }
    catch(err)  { print( { "name"  : "GetVideoInfo()", "error" : err } )                    }

    // get the clock source
    try         { GetProperty("clock_source");                                              }
    catch(err)  { print( { "name"  : "GetProperty(clock_source)", "error" : err } )         }

    // get the current time and date
    try         { GetLocalTime();                                                           }
    catch(err)  { print( { "name"  : "GetLocalTime()", "error" : err } )                    }

    // get the current uptime
    try         { GetCpuTime();                                                             }
    catch(err)  { print( { "name"  : "GetCpuTime()", "error" : err } )                      }

    // get the tv name
    try         { GetTVname();                                                              }
    catch(err)  { print( { "name"  : "GetTVname()", "error" : err } )                       }        

    // get the current channel
    try         { GetCurrentChannel();                                                      }
    catch(err)  { print( { "name"  : "GetCurrentChannel()", "error" : err } )               }    


    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //      Get the pro:centric server info.  This must be the last function in the routine, as it calls setDefaults(),     //
    //      which is the next step in the initialization routine.                                                           //
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    try         { GetProCentricServer();                                                    }
    catch(err)  { print( { "name"  : "GetProCentricServer()", "error" : err } )             }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      This is called in the onSuccess callback of GetProcentricServer().  It sets all the default settings for the TV.                    //
//      It is chained in the callback functions of the hcap calls.                                                                          //
//      It is importatnt that the program flow is not changed.                                                                              //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function setDefaults() {    // STEP 3
    // Enable the Failsafe Reboot
    if (!localStorage.getItem("failsafe")) {
        localStorage.failsafe = 1;
    }

    // Set the power mode to 2 if not defined - warm mode with fetch at a power cycle.  Otherwise use the stored setting
    try         { SetPowerMode();                                                           }
    catch(err)  { print( { "name"  : "SetPowerMode()", "error" : err } )                    }

    // Set Aspect Ratio to Original
    try         { SetAspectRatio(6);                                                        }
    catch(err)  { print( { "name"  : "SetAspectRatio()", "error" : err } )                  }

    // Enable Just Scan
    try         { SetAspectRatio(9);                                                        }
    catch(err)  { print( { "name"  : "SetAspectRatio()", "error" : err } )                  }

    // Set the Display Resolution
    try         { SetProperty("display_resolution", "1920x1080");                           }
    catch(err)  { print( { "name"  : "SetProperty(display_resolution)", "error" : err } )   }

    // Set the Volume
    try         { SetVolume(0);                                                             }
    catch(err)  { print( { "name"  : "SetVolume()", "error" : err } )                       }

    // Style the failsafe button
    try         { SetFailsafe();                                                            }
    catch(err)  { print( { "name"  : "SetFailsafe()", "error" : err } )                     }


    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //      Open the UDP Socket.  This must be the last function in the routine, as it calls restore(),                     //
    //      which is the next step in the initialization routine.                                                           //
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    try         { OpenUDP(udpPort);                                                         }
    catch(err)  { print( { "name"  : "OpenUDP(6868)", "error" : err } )                     }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      This is called in the onSuccess callback of OpenUDP().  It retunes the last channel, registers the TV, and times out the portal.    //
//      It is chained in the callback functions of the hcap calls.                                                                          //
//      It is importatnt that the program flow is not changed.                                                                              //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function restore() {        // STEP 4
    // Retune the last channel
    try         { Retune();
                  setTimeout(function() { if (tuneSuccessful == 0) { Retune(); } }, 4000);  }
    catch(err)  { print( { "name"  : "Retune()", "error" : err } )                          }
    
    // Register the TV
    try         { RegisterTV();                                                             }
    catch(err)  { print( { "name"  : "RegisterTV()", "error" : err } )                      }
    
    // get the current uptime
    try         { TimerPortal(30);                                                          }
    catch(err)  { print( { "name"  : "TimerPortal()", "error" : err } )                     }
}


// Event listeners

window.onload = function () {
    initialize();
};

document.addEventListener(
    "power_mode_changed",
        function() {
            print( { "name"  : "Power Mode", "message" : "Changed" } );
            
            if (!localStorage.getItem("udpFlag")) {
                localStorage.udpFlag = "0";
            }

            var udpFlag = localStorage.getItem("udpFlag");

            GetPowerMode();

            setTimeout(
                function() {
                    if (tv.power_mode  == 1 && udpFlag == "0") {    // power mode is normal (on) and the command did not come via udp (this means it was IR)
                        Retune();

                        localStorage.setItem("udpFlag", "0");

                        setTimeout(
                            function() {
                                if (tuneSuccessful == 0) {
                                    Retune();
                            }
                        }, 2000);

                    }
                    else {                                          // power mode is warm (off) or the command came via udp
                        localStorage.setItem("udpFlag", "0");
                    }
            }, 1000);
        },
        false
);

document.addEventListener(
    "channel_changed", 
    function (param) {
        if (param.result == true) {
            tuneSuccessful = 1;
            
            print( { "name"  : "Channel Change", "message" : "Successful"       } );
            
            GetCurrentChannel();
            GetVideoInfo();
        }
        else {
            tuneSuccessful = 0;

            print( { "name"  : "Channel Change", "error" : param.errorMessage   } );
        }
    },
    false
);

document.addEventListener(
    "udp_data_received",
    function(param) {
        var lead = param.data;

        switch (lead.substring(0,2)) {
            case "00":      // Reboot
                reboot();
                
                break;

            case "01":      // Power state change
                switch (lead.substring(3,4)) {
                    case "0":                           // Turn off to warm mode
                        hcap.power.setPowerMode(
                            {
                                "mode" : hcap.power.PowerMode.WARM,
                                "onSuccess" : function() {
                                    print( { "name"  : "Power Mode", "message" : "set to WARM"  } );
                                    localStorage.setItem("udpFlag", "1");
                                },
                                "onFailure" : function(f) {
                                    print( { "name"  : "Power Mode", "error" : f.errorMessage   } );
                                }
                            }
                        );
                        break;
                    
                    case "1":                           // Turn on
                        hcap.power.setPowerMode(
                            {
                                "mode" : hcap.power.PowerMode.NORMAL,
                                "onSuccess" : function() {
                                    print( { "name"  : "Power Mode", "message" : "set to NORMAL"    } );
                                    localStorage.setItem("udpFlag", "1");
                                },
                                "onFailure" : function(f) {
                                    print( { "name"  : "Power Mode", "error" : f.errorMessage       } );
                                }
                            }
                        );
                        break;
                }
                break;

            case "02":      // Set Volume
                level = Number(lead.substring(3,5));

                SetVolume(level);

                break;

            case "03":      // RF tune
                var tuneMajor, tuneMinor;
                var tuneNumber      = lead.substring (3);                       // strip the header
                len                 = tuneNumber.length;                        // Get the length of what is left
                var i               = len;

                
                do {
                    digit           = tuneNumber.substring(i-1, i);             // get the character to test - working backwards from the end
                    if (digit == ".") {                                         // See if it is the "." then you have all the minor channel digits
                        tuneMinor   = Number(tuneNumber.substring(i));
                        tuneMajor   = Number(tuneNumber.substring(0, i-1));     // what is left is the Major channel number
                        i           = 0;                                        // Store the varaibles and Leave you are done
                    }
                    i--;
                } while(i > 0);
                
                localStorage.setItem("StorType", "RF");                         // Store the type of last channel
                localStorage.setItem("StorMajor", tuneMajor);                   // save the Major channel to restore on reboot
                localStorage.setItem("StorMinor", tuneMinor);                   // save the Minor channel to restore on reboot
                
                TuneRF(tuneMajor, tuneMinor);
                
                break;

            case "04":      // IP tune
                var tuneIp, tunePort;
                var tuneAddress     = lead.substring (3);                       // strip the header
                len                 = tuneAddress.length;                       // Get the length of what is left
                var i               = len;
                
                do {
                    digit           = tuneAddress.substring(i-1, i)             // get the character to test - working backwards from the end
                    if (digit == ":") {                                         // See if it is the ":" then you have all the port digits
                        tunePort    = Number(tuneAddress.substring(i));
                        tuneIp      = tuneAddress.substring(0, i-1);
                        i           = 0;                                        // Store the varaibles and Leave you are done
                    }
                    i--;
                } while (i > 0);
                
                localStorage.setItem("StorType", "IP");                         // Store the type of last channel
                localStorage.setItem("StorIp", tuneIp);                         // save the IP to restore on reboot
                localStorage.setItem("StorPort", tunePort);                     // save the port to restore on reboot
                
                TuneIP(tuneIp, tunePort);
                
                break;

            case "07":      // Set time  07:YYYY-MM-DD:HH:MM:SS-OOOD
                var Year        = Number(lead.substring (3,7));
                var Month       = Number(lead.substring (8,10));
                var Day         = Number(lead.substring (11,13));
                var Hour        = Number(lead.substring (14,16));
                var Minute      = Number(lead.substring (17,19));
                var Second      = Number(lead.substring (20,22));
                var GMToffset   = Number(lead.substring (22,26));
                var Daylight    = lead.substring (26,27);
                
                if (Daylight == "D") {
                    Daylight = true
                }
                else{
                    Daylight =false
                };

                hcap.time.setLocalTime({
                    "year"              : Year,
                    "month"             : Month,
                    "day"               : Day,
                    "hour"              : Hour,
                    "minute"            : Minute,
                    "second"            : Second,
                    "gmtOffsetInMinute" : GMToffset,
                    "isDaylightSaving"  : true,
                    "onSuccess" : function() {
                        print( { "name"  : "Set Time", "message" : "Time set from the Server"   } );
                    },
                    "onFailure" : function(f) {
                        print( { "name"  : "Set Time", "error" : f.errorMessage  } );
                    }
                });

                break;

            case "09":      // TV name:channel  09:TV:DSCHD
                var tvName = lead.substring(3);    //strip the header
                
                SetTVname(tvName);                 // TV:DSCHD
                
                break;

            case "10":      // Closed captioning
                if(lead.substring(3,4) == '0') {
                    hcap.media.setSubtitleOn({ "subtitleOn" : false });
                }
                else {
                    hcap.media.setSubtitle({
                        "type" : hcap.Media.SubtitleType.CLOSED_CAPTION,
                        "index" : 0
                    }
                );
                    hcap.media.setSubtitleOn({ "subtitleOn" : true });
                }
                
                break;

            case "15":      // Toggle the Portal page visibility
                if(lead.substring(3,4) == '0') {
                    HidePortalWindow();
                }
                else {
                    ShowPortalWindow();
                }
                
                break;

            case "16":      // Toggle the TV Name visibility
                localStorage.showName = lead.substring(3,4);
                
                HidePortalWindow();
                
                break;

            case "17":      // Power Mode Always On
                if(lead.substring(3,4) == '0') {
                    powerMode = "2";                //User control of power
                } 
                else {
                    powerMode = "10";               //Always on
                }
                localStorage.setItem("LocalPowerMode", powerMode);
                
                SetPowerMode();
                
                break;

            case "18":      // Resize video (video wall) 18:x:y:width:height:
                var i, start = 3, index = 0;
                var value = [];
                
                for(i = 3; i < lead.length && index < 4; ++i) {
                    value[index] = Number(lead.substring(start, i));
                    if(lead.substring(i, i+1) == ":") {
                        ++index;
                        start = i + 1;
                    }
                }
                hcap.video.setVideoSize(
                    {
                        "x"         : value[0],
                        "y"         : value[1],
                        "width"     : value[2],
                        "height"    : value[3]
                    }
                );
                break;

            case "19":      // Input select 19:00 or 19:10 or 19:11
                var input = lead.substring(3,4);
                var index = Number(lead.substring(4,5));
                var type;
                
                switch (input) {
                    case "0":   //TV
                        type = hcap.externalinput.ExternalInputType.TV;
                        break;
                    
                    case "1":   //HDMI
                        type = hcap.externalinput.ExternalInputType.HDMI;
                        break;
                    
                    case "2":   //SVIDEO
                        type = hcap.externalinput.ExternalInputType.SVIDEO;
                        break;
                    
                    case "3":   //COMPOSITE
                        type = hcap.externalinput.ExternalInputType.COMPOSITE;
                        break;
                    
                    case "4":   //COMPONENT
                        type = hcap.externalinput.ExternalInputType.COMPONENT;
                        break;
                    
                    case "5":   //RGB
                        type = hcap.externalinput.ExternalInputType.RGB;
                        break;
                    
                    case "6":   //SCART
                        type = hcap.externalinput.ExternalInputType.SCART;
                        break;
                    
                    case "7":   //USB
                        type = hcap.externalinput.ExternalInputType.USB;
                        break;
                    
                    case "8":   //OTHERS
                        type = hcap.externalinput.ExternalInputType.OTHERS;
                        break;
                    
                    default:    //Default to TV
                        type = hcap.externalinput.ExternalInputType.TV;
                        break;
                    break;
                }

                hcap.externalinput.setCurrentExternalInput(
                    {
                        "type"  : type,
                        "index" : index,
                            "onSuccess" : function() {
                                print( { "name"  : "External Input", "message" : "Set to " + type.substring(38) } );
                            },
                            "onFailure" : function(f) {
                                print( { "name"  : "External Input", "error" : f.errorMessage                   } );
                            }
                    }
                );
                break;

            case "20":      // Misc functions 
                switch (lead.substring(3,4)) {
                    case "0":                       // Reload the page
                        CloseUDP();
                        location.reload(true);
                        
                        break;
                    
                    case "1":                       // Toggle Debugging
                        ToggleDebug();
                        
                        break;

                    case "2":                       // Toggle Failsafe Reboot
                        ToggleFailsafe()
                        
                        break;
                }
                break;
        }
    },
    false
);
